#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from constants import *

import random
import os
import pygame
from pygame import *

from utility import *
import data

# Defines the fruits and their respective files
fruits = {'apple'   :  'apple.txt', \
          'orange'  :  'orange.txt',\
          'banana' : 'banana.txt', \
          'strawberry' : 'strawberry.txt', \
          'rasberry' : 'rasberry.txt', \
          'cherry' : 'cherry.txt', \
          'kiwi' : 'kiwi.txt', \
          'lemon' : 'lemon.txt', \
          'lime' : 'lime.txt', \
          'apricot' : 'apricot.txt', \
          }

def random_fruit():
    return random.choice(fruits.keys())

class FruitSprite(pygame.sprite.Sprite):
    def __init__(self, playerid=0, grid=None, type=None):
        pygame.sprite.Sprite.__init__(self)
        
        self.rect = Rect(0,0,CELL_X,CELL_Y)
        
        self.type = type
        if self.type == None:
            self.type = random_fruit()
    
        self.id = playerid
        
        if grid == None:
            self.grid = random_grid()
        else:
            self.grid = grid
        
        self.set_grid(self.grid)
        
        self.support = 0
        self.juice = 0
        
        self.initialize()
        
        self.fall = 0
        self.selected = False
        self.troubled = False
        self.troubled_drawn = False

    def initialize(self):
        fp = data.load(fruits[self.type])
        filename = fp.readline().strip()
        self.support = int(fp.readline().strip())
        self.juice = int(fp.readline().strip())
        fp.close()
        
        #imagepath = data.filepath(os.path.join(FRUIT_DIR,filename))
        image = load_image(os.path.join(FRUIT_DIR,filename))
        
        self.image = pygame.transform.scale(image,self.rect.size)
        self.image.convert()
        
        self.backup = self.image.copy()
    
    def set_grid(self, value, other_fruits=None):
        allow = True
        if other_fruits != None:
            allow = self.check_placement(other_fruits,self.value)        
        if allow:
            self.grid = value
            self.rect.center = toxy(self.grid,self.id)

    def check_placement(self,other_fruits,grid):
        center_buffer = self.rect.center
        self.rect.center = toxy(grid)
        ret = len(pygame.sprite.spritecollide(self,other_fruits,False)) > 1
        self.rect.center = center_buffer
        return ret

    def update_falling(self,other_fruits):
        # Don't fall if on the bottom
        if self.rect.bottom >= bottom() or self.selected:
            self.fall = 0
            return
        
        center_buffer = self.rect.center
        
        self.fall = self.fall + FALL_SPEED
        self.rect.bottom = self.rect.bottom + self.fall
        if len(pygame.sprite.spritecollide(self,other_fruits,False)) > 1:
            self.fall = 0
        
        self.rect.center = center_buffer
    
    def set_selected(self,value=True):
        self.selected = value
    
    def set_troubled(self):
        self.troubled = True
    
    def get_support(self):
        return self.support
    
    def get_juice(self):
        return self.juice
    
    def is_selected(self):
        return self.selected
    
    def get_grid(self):
        return togrid(self.rect.center, self.id)
      
    def set_xy(self,xy):
        self.rect.center = xy
    
    def update(self):
        if self.selected:
            # Movement is handled in set_xy
            pass
        else:
            # Falling animation
            self.rect.bottom = min(self.rect.bottom + self.fall,bottom())
        
        if self.troubled:
            if self.troubled_drawn:
                self.troubled_drawn = False
                self.image = self.backup.copy()
            else:
                self.troubled_drawn = True
                self.image.fill(self.image.get_colorkey())
        
        
        
        
    