#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from constants import *
import client_net

import data
from play import *

import cmd

class JuicerCmd(cmd.Cmd):
    def __init__(self):
        cmd.Cmd.__init__(self)
        self.prompt = ">"
        self.intro  = INTRO
        
        self.elf_sound_flag = True
        self.sound_flag = True
        self.server_name = DEF_SERVER
        self.name = None
        self.server = None
        
    def do_exit(self, args):
        """Exits from the console"""
        if self.server:
            self.do_disconnect('')
        return -1
    
    def do_quit(self, args):
        """Exits from the console"""
        if self.server:
            self.do_disconnect('')
        return -1
    
    def do_play(self,args):
        """Instantly starts a time-trial game (single-player)"""
        print "Starting a time-trial game..."
        scores = play(1,self.sound_flag,self.elf_sound_flag)
        if self.server != None and len(scores) > 0:
            if len(scores) == 1:
                print client_net.send_score(self.server,self.name,scores[0])
        
    def do_about(self,args):
        """Explains this game a bit"""
        print ABOUT
        
    def do_controls(self,args):
        """Explains game controls..."""
        print CONTROLS
        
    def do_elf_sound(self,args):
        """Turns on and off the elf giggle (very annoying...).  Type 'on' or 'off'."""
        if len(args) > 0:
            self.elf_sound_flag = (args.lower() == 'on')
        if self.elf_sound_flag:
            print "Elf laughing is on"
        else:
            print "Elf laughing is off"
            
    def do_sound(self,args):
        """Turns on and off all sound.  Type 'on' or 'off'."""
        if len(args) > 0:
            self.sound_flag = (str(args).lower() == 'on')
        if self.sound_flag:
            print "Sound enabled"
        else:
            print "Sound disabled"
            
    def do_connect(self,args):
        """Connects to a Juicer server for score tracking.  Defaults to the
Juicer server at juicer.rainbow-100.com"""
        if self.name == None:
            print "Please enter your username first"
        elif self.server != None:
            print "Already connected.  Please use 'disconnect' first"
        else:
            if len(args) > 0:
                self.server_name = args
            print "Connecting to %(server)s, standby..." % {'server': self.server_name}
            self.server = client_net.connect(self.server_name, self.name)
            if self.server == None:
                print "Connection to %(server)s failed" % {'server': self.server_name}
                print "Error message: %(message)s" % {'message': client_net.get_error()}
            else:
                print "Now online"

    def do_disconnect(self,args):
        """Disconnects from a Juicer server"""
        if self.server == None:
            print "No need for that"
        else:
            try:
                self.server.send('QUIT\n\r')
                self.server.close() 
            finally:
                print "Server connection closed"
                self.server = None

    def do_name(self,args):
        """Sets your username for online play. Type 'name' followed by a space and your username"""
        if len(args) > 0:
            self.name = args
            print "Name set to %(name)s" % {'name':self.name}

def main():
    
    cmd_proc = JuicerCmd()
    cmd_proc.cmdloop()
    
    # heplay()
