#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

import SocketServer, time, select, sys
from threading import Thread
from serverdb import *
from net_constants import *

COMMAND_HELLO = 1
COMMAND_QUIT  = 2

class JuicerServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):

    daemon_threads = True
    allow_reuse_address = True

    def __init__(self, server_address, RequestHandlerClass, message=''):
        SocketServer.TCPServer.__init__(self, server_address, RequestHandlerClass)
        self.message = message

class JuicerRequestHandler(SocketServer.BaseRequestHandler):

    def __init__(self, request, client_address, server):
        SocketServer.BaseRequestHandler.__init__(self, request, client_address, server)

    def handle(self):
        #self.request.send(self.server.message)

        read_ready, write_ready, in_error = select.select([self.request], [], [], None)

        text = ''
        done = False
        while not done:

            if len(read_ready) == 1 and read_ready[0] == self.request:
                data = self.request.recv(512)
                if not data:
                    break
                elif len(data) > 0:    
                    text = text + data
                    while text.find('\n') > 1:
                        line, text = text.split('\n',1)                               
                        response = 'Command not understood'
                        line = line.strip()
                        
                        if line.startswith('NAME: ') and line.find('SCORE: ') > 0:
                            if self.processAndCommitScore(line):
                                response = 'OK'
                            else:
                                response = 'Failed to commit score'
                        elif line.startswith('NAME: '):
                            response = 'OK'
                        elif line.startswith('QUIT'):
                            done = True
                            response = 'GOODBYE'
                        
                        self.request.send(response+'\r\n')
        self.request.close()

    def processAndCommitScore(self,line):
        ret = False
        name_start = len('NAME: ')
        name_end = line.find('SCORE: ')-1
        name = None
        if name_end > name_start:
            name = line[name_start:name_end]
            
        score_start = name_end + 1 + len('SCORE: ')
        score = None
        if score_start > name_end:
            try:
                score = float(line[score_start:].strip())
            except ValueError:
                score = None
        
        if score != None and name != None:
            db = ServerDB()
            db.update_user_time(name,score)
            ret = True
        
        return ret

    def finish(self):
       """Nothing"""

class JuicerServerThread(Thread):
    def __init__(self):
        Thread.__init__(self)
        
        self.server = JuicerServer(('', SERVER_PORT), JuicerRequestHandler, 'welcome')
        
    def run(self):
        self.server.serve_forever()
        
    def stop_serving(self):
        self.server.server_close()
        self.server.socket.close()


