#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

import sqlite3

DEFAULT_DB_FILE = "juicer server.database"

class ServerDB:
    def __init__(self,file=DEFAULT_DB_FILE):
        self.file = file
        
        self.conn = sqlite3.connect(self.file)
        self.cur = self.conn.cursor()
        
        self.cur.execute('''create table if not exists juicer 
                            (name text, best float, wins integer, losses integer, games integer)''')
        self.cur.execute('''create view if not exists orderedtimes as
                            select name, best from juicer order by best desc''')
        self.cur.execute('''create view if not exists orderedwins as
                            select name, wins from juicer order by wins desc''')
        self.cur.execute('''create view if not exists orderedgames as
                            select name, games from juicer order by games desc''')

    def update_user_time(self,name,time):
        self.update_user_games(name)
        
        self.cur.execute('''select best from juicer where name=?''', 
                         (name,))
        val = -1.0
        for row in self.cur:
            tbl = row[0]
            val = max(time,tbl)
        
        if val == -1.0:
            # User doesn't exist
            self.create_user(name)
            val = time
        
        self.cur.execute('''update juicer set best=? where name=?''',
                         (val,name))
        self.conn.commit()
        
    def create_user(self,name):
        self.cur.execute('''insert into juicer values(?,?,?,?,?)''',(name,0.0,0,0,0))
    
    def update_user_win(self,name):
        self.update_user_games(name)
        
        self.cur.execute('''select wins from juicer where name=?''', 
                         (name,))
        val = -1
        for row in self.cur:
            val = row[0]
        
        if val == -1:
            # User doesn't exist
            self.create_user(name)
            val = 0
        
        val = val + 1
        
        self.cur.execute('''update juicer set wins=? where name=?''',
                         (val,name))
        self.conn.commit()

    def update_user_loss(self,name):
        self.update_user_games(name)
        
        self.cur.execute('''select losses from juicer where name=?''', 
                         (name,))
        val = -1
        for row in self.cur:
            val = row[0]
        
        if val == -1:
            # User doesn't exist
            self.create_user(name)
            val = 0
        
        val = val + 1
        
        self.cur.execute('''update juicer set losses=? where name=?''',
                         (val,name))
        self.conn.commit()
    
    def update_user_games(self,name):
        self.cur.execute('''select games from juicer where name=?''', 
                         (name,))
        val = -1
        for row in self.cur:
            val = row[0]
        
        if val == -1:
            # User doesn't exist
            self.create_user(name)
            val = 0
        
        val = val + 1
        
        self.cur.execute('''update juicer set games=? where name=?''',
                         (val,name))
    
    def get_user_time(self,name):
        self.cur.execute('''select best from juicer where name=?''',
                         (name,))
        val = 0.0
        for row in self.cur:
            val = row[0]
        return val
        
    def get_user_wins(self,name):
        self.cur.execute('''select wins from juicer where name=?''',
                         (name,))
        val = 0
        for row in self.cur:
            val = row[0]
        return val

    def get_scorers(self,count=10):
        self.cur.execute('''select * from orderedtimes limit ?''',(count,))
        ret = []
        for row in self.cur:
            ret.append(row)
        return ret
    
    def get_winners(self,count=10):
        self.cur.execute('''select * from orderedwins limit ?''',(count,))
        ret = []
        for row in self.cur:
            ret.append(row)
        return ret
    
    def get_games(self,count=10):
        self.cur.execute('''select * from orderedgames limit ?''',(count,))
        ret = []
        for row in self.cur:
            ret.append(row)
        return ret
        
