#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from threading import Thread
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from net_constants import *

from serverdb import *

PAGE_HEADER = """<html><title>Juicer Worldwide Standings</title>
<body>
    <center><h1>Juicer: Twisted Little Elves</h1><br>
    <h2>Worldwide Leader Board</h2><br><br>
"""

PAGE_FOOTER = """<br>Juicer is brought to you by Printstar<br>
for PyWeek September 2007.<br>
Juicer Server - Copyright 2007 Jeffrey Armstrong<br>
</body></html>"""    

class WebstatHandler(BaseHTTPRequestHandler):

    def do_GET(self):
        try:
            self.send_response(200)
            self.send_header('Content-type',	'text/html')
            self.end_headers()
            
            self.wfile.write(PAGE_HEADER)
            
            self.wfile.write(self.gen_time_table())        
            self.wfile.write(self.gen_games_table())
            #self.wfile.write(self.gen_win_table())        
           
            self.wfile.write(PAGE_FOOTER)
            return
                
        except IOError:
            self.send_error(404,'File Not Found: %s' % self.path)
     
    def do_POST(self):
        pass

    def gen_time_table(self):
        ret = "<h3>Best Times</h3><br>"
        db = ServerDB()

        if db != None:
            ret = ret + self.gen_table(db.get_scorers(),"Time")
        return ret

    def gen_win_table(self):
        ret = "<h3>Most Wins</h3><br>"
        db = ServerDB()

        if db != None:
            ret = ret + self.gen_table(db.get_scorers(),"Number of Wins")
        return ret
    
    def gen_games_table(self):
        ret = "<h3>Most Games Played</h3><br>"
        db = ServerDB()

        if db != None:
            ret = ret + self.gen_table(db.get_games(),"Number of Games")
        return ret
      
    def gen_table(self, values, val_header):
        ret = "<table border = 1>"
        
        ret = ret + "<tr><td>Name</td><td>" + val_header + "</td></tr>"
        
        for row in values:
            if len(row) >= 2:
                ret = ret + "<tr><td>"
                ret = ret + row[0]
                ret = ret + "</td><td>"
                ret = ret + str(row[1])
                ret = ret + "</td></tr>"
        
        return ret+"</table>"
        
class WebServer(Thread):
    def __init__(self):
        Thread.__init__(self)
        self.server = HTTPServer(('', HTTP_PORT), WebstatHandler)
    
    def run(self):
        self.server.serve_forever()
    
    def stop_serving(self):
        self.server.server_close()
        self.server.socket.close()

