#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from constants import *

import data

import pygame
from pygame import *

from utility import *
from player import *
from press import *
from fruit import *
from populate import *
from game_update import *
from juice import *
from message import *
from elf import *

def play(num_players=1,sound=True,elf_sound=True):
     
    pygame.init()
    
    # Sound init
    if pygame.mixer and not pygame.mixer.get_init():
        print 'Problem with the sound...'
        pygame.mixer = None
        sound = False
    elif sound:
        pygame.mixer.init()
    
    screen = pygame.display.set_mode((PLAY_WIDTH*num_players,PLAY_HEIGHT))
    pygame.display.set_caption(TITLE)
    
    # Screen Drawing...
    bkgd = load_image(BKGD_IMG,None)
    for x in range(0,num_players):
        screen.blit(bkgd,(x*PLAY_WIDTH,0))

    #screen.blit(bkgd,(0,0))
    pygame.display.flip()
    
    # Initialize clock
    clock = pygame.time.Clock()
    
    # Initialize mouse
    pygame.mouse.set_visible(True)
    
    # Initialize font
    font = pygame.font.Font(data.filepath(FONT_NAME),32)
    
    # Initialize players
    myplayer = PlayerSprite("Local",True)
    
    # Initialize juicing presses
    press_sprites = pygame.sprite.RenderUpdates()
    for x in range(0,num_players):
        press_sprites.add(PressSprite(x))
    
    # And their elves
    elf_sprites = pygame.sprite.RenderUpdates()
    for x in range(0,num_players):
        elf_sprites.add(ElfSprite(x))
    if sound:
        elf_giggle = pygame.mixer.Sound(data.filepath('giggle.ogg'))
        elf_giggle.set_volume(0.4)
    
    # Build the lists
    player_sprites = pygame.sprite.RenderUpdates(myplayer)
    
    # Fruit sprites
    fruit_sprites = populate(45, 0)
    if sound:
        pop = pygame.mixer.Sound(data.filepath('pop.ogg'))
    
    # Juice sprites
    juice_sprites = pygame.sprite.RenderUpdates()
    for i in range(0,num_players):
        juice_sprites.add(JuiceSprite(i))
    
    # Scores
    scores = []
    for i in range(0,num_players):
        scores.append(0.0)
    
    # Message sprites
    message_sprites = pygame.sprite.RenderUpdates()
    
    # start the music
    if sound:
        music = pygame.mixer.Sound(data.filepath('music.ogg'))
        music.set_volume(0.3)
        music.play(-1)
    
    running = True
    start_time = pygame.time.get_ticks()
    press_time = 0
    wave_time = 0
    juice_inc_time = 0
    xmy = (0,0)
    game_done = False
    while running:
        clock.tick(FPS)
        
        # Check what time it is...
        current_time = pygame.time.get_ticks()
        
        # Clear the player sprites
        fruit_sprites.clear(screen,bkgd)
        juice_sprites.clear(screen, bkgd)
        elf_sprites.clear(screen,bkgd)
        player_sprites.clear(screen,bkgd)
        message_sprites.clear(screen,bkgd)
    
        # Are the presses moving?
        do_presses = current_time-press_time > PMOVE_COUNT
        press_sprites.clear(screen, bkgd)
        if do_presses:
            for p in press_sprites:
                if not p.done():
                    p.move()
            for e in elf_sprites:
                e.run_anim()
            if sound and elf_sound:
                elf_giggle.play()
                
            press_time = current_time
        
        # Event handling
        for event in pygame.event.get():
            if game_done:
                if event.type == KEYUP or event.type == pygame.QUIT:
                    running = False
            else:
                if event.type == pygame.QUIT: 
                    running = False
                elif event.type == pygame.MOUSEBUTTONUP:
                    mxy = pygame.mouse.get_pos()
                
                    # Handle right click to grab fruit
                    if event.button == 3:
                        myplayer.check_fruit(True)
                    
                    myplayer.set_grid_dest(togrid(mxy,0))
    
        # Update players
        all_dead = True
        dead_count = 0
        for player in player_sprites:
            
            if player.get_done():
                continue
            
            if player.check_fruit():
                if player.arrived():
                    
                    player.check_fruit(False)
                    if player.get_fruit() == None:
                        # Find which fruit is selected
                        possibles = pygame.sprite.spritecollide(player, fruit_sprites, False)
                        for x in possibles:
                            if x.get_grid() == player.get_grid():
                                x.set_selected(True)
                                player.set_fruit(x)
                                break
                    else:
                        if len(pygame.sprite.spritecollide(player.get_fruit(),fruit_sprites,False)) <= 1:
                            player.get_fruit().set_selected(False)
                            player.set_fruit(None)
            if not player.get_dead():
                all_dead = False
            elif not game_done:
                dead_count = dead_count + 1
                message_sprites.add(display_dead(player.get_id(),current_time-start_time))
                scores[player.get_id()] = float(current_time-start_time)/1000.0
                
            if player.get_dead():
                player.set_done()
                player.set_grid_dest((player.get_grid()[0],0))
                for e in elf_sprites:
                    if e.get_id() == player.get_id():
                        e.done()
            
        if num_players > 1 and dead_count == num_player - 1 and not game_done:
            for player in player_sprites:
                if not player.get_dead():
                    message_sprites.add(display_win(player.get_id()))
                    game_done = True
        
        if all_dead:
            game_done = True
                
        player_sprites.update()
        
        
        # Update all fruit
        for fruit in fruit_sprites:
            fruit.update_falling(fruit_sprites)

        fruit_sprites.update()
        
        # Update presses
        if do_presses:
            press_sprites.update()
        
        # Finally, update the game
        juices = game_update(player_sprites, \
                             press_sprites,  \
                             fruit_sprites,  \
                             juice_sprites)
        
        # Update all the juice levels
        for x in juice_sprites:
            if juices[x.get_id()] > 0:
                x.inc_juice(juices[x.get_id()])
                if sound:
                    pop.play()
        if current_time - wave_time > WAVE_COUNT:
            wave_time = current_time
            for x in juice_sprites:
                x.waves()
        if current_time - juice_inc_time > JUICE_INC_COUNT:
            juice_inc_time = current_time
            for x in juice_sprites:
                x.update_level()
        
        elf_sprites.update()
        
        juice_sprites.update()
        message_sprites.update()
        
        dirty = fruit_sprites.draw(screen) + \
                player_sprites.draw(screen) 
        
        dirty = dirty + press_sprites.draw(screen)
        dirty = dirty + elf_sprites.draw(screen)
        dirty = dirty + juice_sprites.draw(screen)
        dirty = dirty + message_sprites.draw(screen)
        pygame.display.update(dirty)
        
    if sound:
        music.stop()
    pygame.display.quit()
    
    return scores