#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from constants import *
import pygame
from pygame import *

from utility import *

class PlayerSprite(pygame.sprite.Sprite):
    def __init__(self, name, local=True, playerid=0):
        pygame.sprite.Sprite.__init__(self)
        
        self.rect = Rect(0,0,CELL_X,CELL_Y)
        self.rect.center  = toxy((0,0),playerid)
        self.image = Surface((self.rect.width,self.rect.height))
        
        self.name = name
        self.local = local
        self.id = playerid
        
        self.images = []
        self.image_i = 0
        self.init_image()
        
        self.gdest = (10,10)
        #self.sdest = toxy(self.gdest, self.id)
        self.set_grid_dest((0,0))
        
        self.holding = None
        self.fruit_handling = False
        
        self.dead = False
        self.done = False
    
    def init_image(self):
        #self.image = Surface((self.rect.width, self.rect.height))
        #self.image.set_colorkey(Color('white'))
        #self.image.fill(Color('white'))
        #self.image.convert()
        for file in PLAYER_FRAMES:
            img = load_image(file)
            self.images.append(pygame.transform.scale(img,self.rect.size))
        self.image = self.images[self.image_i]
        
    def set_grid_dest(self,gpt):
        if gpt != self.gdest:
            self.gdest = gpt
            self.sdest = toxy(self.gdest, self.id)
    
    def arrived(self):
        return self.rect.center == self.sdest
    
    def set_fruit(self, fruit=None):
        self.holding = fruit
    
    def get_fruit(self):
        return self.holding
    
    def check_fruit(self, value=None):
        if value != None:
            self.fruit_handling = value
        return self.fruit_handling
    
    def get_grid(self):
        return togrid(self.rect.center, self.id)

    def get_dead(self):
        return self.dead
    
    def set_dead(self,value):
        self.dead = value

    def set_done(self):
        self.done = True

    def get_done(self):
        return self.done

    def get_id(self):
        return self.id

    def update(self):
        
        # Update to the next position
        if not self.arrived():
             
            diffx = self.sdest[0]-self.rect.center[0]
            nx = self.sdest[0]
            if abs(diffx) > XSPEED:
                nx = self.rect.center[0] + sign(diffx)*XSPEED

            diffy = self.sdest[1]-self.rect.center[1]
            ny = self.sdest[1]
            if abs(diffy) > YSPEED:
                ny = self.rect.center[1] + sign(diffy)*YSPEED
            
            self.rect.center = (nx,ny)
            
            if self.holding != None:
                self.holding.set_xy(self.rect.center)
        
        # Animation
        self.image_i = self.image_i + 1
        if self.image_i >= len(self.images):
            self.image_i = 0
        self.image = self.images[self.image_i]
            
        