#    Juicer: Twisted Little Elves
#    Copyright (C) 2007 Jeffrey Armstrong
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

from constants import *
import pygame
from pygame import *

from utility import *

class PressSprite(pygame.sprite.Sprite):
    def __init__(self, playerid=0):
        pygame.sprite.Sprite.__init__(self)
        
        self.id = playerid
        
        self.rect = Rect(PRESS_X_OFFSET,PLAY_HEIGHT-(PRESS_Y_OFFSET+PRESS_HEIGHT),PRESS_WIDTH,CELL_Y)
        
        self.init_image()
        
        self.moved = False
        self.grid = (0,GRID_Y_CNT)
        self.support = 0
        self.pressure = 0
        
        self.stopall = False
    
    def init_image(self):
        self.image = load_image(PRESS_IMG)
        self.image.convert()
        self.rect = self.image.get_rect()
        self.rect.bottomleft = (PRESS_X_OFFSET,PLAY_HEIGHT-(PRESS_Y_OFFSET+PRESS_HEIGHT))
        
    def move(self):
        self.moved = True
    
    def done(self):
        return self.grid == 0
        
    def get_grid(self):
        return self.grid
    
    def get_gy(self):
        return self.grid[1]
    
    def get_gx(self):
        return self.grid[0]
    
    def get_touching_fruits(self,fruits):
        center_buffer = self.rect.center
        self.rect.bottom = self.rect.bottom + 1
        hitting = pygame.sprite.spritecollide(self,fruits,False)
        self.rect.center = center_buffer
        return hitting
    
    def stop(self):
        self.stopall = True
    
    def check_for_support(self,fruits):
        self.support = 0
        for x in self.get_touching_fruits(fruits):
            self.support = self.support + x.get_support()
    
    def get_popped(self,fruits):
        if self.pressure > self.support:
            self.pressure = 0
            return self.get_touching_fruits(fruits)
        else:
            return []
    
    def get_trouble(self,fruits):
        touching = self.get_touching_fruits(fruits)
        if len(touching) == 0:
            return []
        
        if self.pressure/len(touching) + TROUBLE > self.support/len(touching):
            return touching
        else:
            return []
    
    def get_pressure(self):
        return self.pressure
        
    def set_proceed(self):
        self.support[self.grid-1] = 0
        self.pressure = 0
    
    def get_bottom(self):
        return self.rect.bottom
    
    def get_id(self):
        return self.id
    
    def update(self):
        if self.moved and not self.stopall:
            self.moved = False
            
            newgrid = togrid(self.rect.bottomleft,self.id)
            
            if self.support > 0:
                self.pressure = self.pressure + PMOVE
            else:
                self.rect.bottom = self.rect.bottom + PMOVE
                self.grid = togrid(self.rect.bottomleft,self.id)
            
            